/*
 * $Id: AlarmMessage.cpp,v 1.1.1.1 2002/09/24 17:40:06 dwalters Exp $
 *
 * Copyright (C) 2002 Dafydd Walters <dwalters@users.sourceforge.net>
 * Modified by Anton Maslovsky <my-zaurus@narod.ru> 2003/01/28
 * 
 * This program may be modified or redistributed under the terms of the GNU
 * General Public License (GPL), version 2 or later. A copy of the GNU GPL 
 * should have accompanied the source code of this program. If not, please 
 * write to the Free Software Foundation Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * This program is distributed without any warranty.
 */ 

#include <qpushbutton.h>
#include <qdatetime.h>
#include <qtimer.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qgroupbox.h>

#include <qpe/sound.h>
#include <qpe/applnk.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/timestring.h>
#include <qpe/fileselector.h>
#include <qpe/resource.h>

// includes for alarm led
#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

#include "sharp_char.h"

// includes for sound
#include <sys/ioctl.h>
#include <linux/soundcard.h>

#include "AlarmMessage.h"
#include "qpealarmclock.h"

AlarmMessage::AlarmMessage(AlarmDetails* pNewAlarm, QWidget* parent, const char* name, bool modal,
                           WFlags fl) :
                           AlarmMessageBase(parent, name, modal, fl)
{
    QImage imgAlarm = Resource::loadImage("qpealarmclock/QpeAlarmClock");
    QPixmap pixAlarm;
    pixAlarm.convertFromImage(imgAlarm.smoothScale(AppLnk::bigIconSize(), AppLnk::bigIconSize()));
    
    pixIcon->setPixmap(pixAlarm);
    
    
    pAlarmClock = (AlarmClock *)parent;
    pAlarm = pNewAlarm;

    connect(btnSnooze, SIGNAL(clicked()), this, SLOT(snoozeClicked()));

    if (!pAlarm->message().isNull())
      lblALARM->setText(pAlarm->message());

    int hour, minute;
    hour = pAlarm->time().hour();
    minute = pAlarm->time().minute();
    if (pAlarmClock->twelveHourClock)
    {
      if (hour < 12)
        lblTime->setText(QString().sprintf("%d:%02d",
          hour == 0 ? 12 : hour, minute) + tr(" AM"));
      else
      lblTime->setText(QString().sprintf("%d:%02d",
          hour == 12 ? 12 : hour - 12, minute) + tr(" PM"));
    }
    else
      lblTime->setText(QString().sprintf("%02d:%02d", hour, minute));

    if (pAlarm->snoozeInterval() > 0)  
      btnSnooze->setText(tr("Snooze (") + QString::number(pAlarm->snoozeInterval()) + tr(" min)"));
    else
      btnSnooze->hide();

    // sound alarm if loud notification is set
    if (pAlarm->notification() == AlarmDetails::loud)
    {
      // calculate time to play this sound
      QTime t;
      t.start();
      playSound(pAlarm->device(), pAlarm->sound());
      
      stop = 0;
      timer = new QTimer(this, "timer");
      timer->start(t.elapsed() + 2000);
    
      connect(timer, SIGNAL(timeout()), this, SLOT(timerTriggered()));
    }

    // blink led
    blinkLed(pAlarm->led());
}

AlarmMessage::~AlarmMessage()
{
  // reset led to default
  blinkLed(LED_COLLIE_1_DEFAULT);
}

void AlarmMessage::blinkLed(int led)
{
  int fd;
  sharp_led_status st;

  fd = open ("/dev/sharp_led", O_WRONLY);
  st.which = SHARP_LED_COLLIE_1;
  st.status = led;
  ioctl(fd, SHARP_LED_SETSTATUS, &st);
}

void AlarmMessage::playSound(const QString& device, const QString& sound)
{
  if (sound == "")
  {
    Sound::soundAlarm();
    return;
  }

  if (!playSoundOnDevice(device, sound))
    Sound::soundAlarm(); // fallback
}


void AlarmMessage::timerTriggered()
{
    if ( stop < 20 ) 
    {
      playSound(pAlarm->device(), pAlarm->sound());
      stop++;
    } 
    else 
    {
      stop = 0;
      timer->stop();
      //delete timer;
    }
}

void AlarmMessage::snoozeClicked()
{
  qDebug("Snoozing alarm: " + pAlarm->toString());

  // reschedule another alarm in snoozeInterval() minuties
  AlarmDetails* pSnoozeAlarm = pAlarmClock->snoozeAlarm();

  pSnoozeAlarm->setAlarm(true,
    QTime::currentTime().addSecs(60 * pAlarm->snoozeInterval()),
    AlarmDetails::once,
    pAlarm->message(),
    pAlarm->notification(),
    pAlarm->led(),
    pAlarm->sound(),
    pAlarm->device(),
    pAlarm->snoozeInterval(),
    pAlarm->weekdaysFlags());

  AlarmMessageBase::accept();
}

#define WAVE_FORMAT_PCM 1
struct WAV_HEADER
{
    char id[4];
    Q_UINT32 size;
    char data[4];
};

struct WAV_DATA
{
    Q_INT16 formatTag;
    Q_INT16 channels;
    Q_INT32 samplesPerSec;
    Q_INT32 avgBytesPerSec;
    Q_INT16 blockAlign;
    Q_INT16 wBitsPerSample;
};

bool AlarmMessage::playSoundOnDevice(const QString& device, const QString& sound)
{
  WAV_HEADER header;
  WAV_DATA data;
  
   // read wav file header
   QFile track(sound);

   if( !track.open( IO_ReadOnly ))
   {
      qDebug("Failed to open the wav file: " + sound);
      return false;
   }

   int length;

    // defaults for buzzer
    int speed = 22050;
    int channels = 1;
    int  format = AFMT_S16_LE; // or AFMT_U8, but has buggy output on the Z

    // adjust params for normal sound device
    if (device != "/dev/sharp_buz")
     {
        const int n = sizeof(header) - sizeof(header.data);
        length = track.readBlock( (char*)&header, n);
        if (length == -1)
        {
           qDebug("Failed to read wav file header");
           return false;
        }

        qDebug(QString("header.id: ") + header.id);
        if (strncmp(header.id,"RIFF",4) == 0)
        {
           char d[4];
     		  if (track.readBlock(d, 4) != 4 )
           {
               qDebug("Failed to read wav file header");
     		      return false;
     		  }
           qDebug(QString("d: ") + d);
     		  if (strncmp(d,"WAVE",4) != 0 )
           {
     		    // skip
     		    if (header.size > 1000000000 || !track.at(track.at() + header.size - 4))
             {
               qDebug("Failed to read wav file header");
     		      return false;
             }
           }

           length = track.readBlock( (char*)&header, n);
           if (length == -1)
           {
             qDebug("Failed to read wav file header");
             return false;
           }

           qDebug(QString("header.id: ") + header.id);
           if (strncmp(header.id,"fmt ",4) == 0)
           {
             // read format tag
     		    if ( track.readBlock((char*)&data,sizeof(data)) != sizeof(data) )
             {
               qDebug("Failed to read wav file header");
     		      return false;
     		    }
           }
        }

        qDebug(QString("data.formatTag: ") + QString::number(data.formatTag));
        if (data.formatTag != WAVE_FORMAT_PCM)
        {
           qDebug("Unsupported wav format");
           return false;
        }

       qDebug(QString("data.samplesPerSec: ") + QString::number(data.samplesPerSec));
       qDebug(QString("data.channels: ") + QString::number(data.channels));
       qDebug(QString("data.wBitsPerSample: ") + QString::number(data.wBitsPerSample));

      qDebug("Prepared file: " + sound);

      speed = data.samplesPerSec;//22050;
      channels = data.channels; //1;
      format = data.wBitsPerSample; //AFMT_U8;//AFMT_S16_LE; // or AFMT_U8, but has buggy output on the Z

    }
    
     int dsp=0;
     signed short soundBuffer[1024];

     dsp = ::open(device, O_WRONLY);
     if( dsp == -1)
     {
       qDebug("Failed to open(" + device + ")");
       return false;
     }

     if( ioctl( dsp, SNDCTL_DSP_SETFMT , &format) == -1)
     {
         qDebug("Failed to ioctl(\"SNDCTL_DSP_SETFMT\")");
         ::close (dsp);
         return false;
     }

     if( ioctl( dsp, SNDCTL_DSP_CHANNELS , &channels) == -1)
     {
         qDebug("Failed to ioctl(\"SNDCTL_DSP_CHANNELS\")");
         ::close (dsp);
         return false;
     }

     if( ioctl( dsp, SNDCTL_DSP_SPEED , &speed) == -1)
     {
         qDebug("Failed to ioctl(\"SNDCTL_DSP_SPEED\")");
         ::close (dsp);
         return false;
     }

   qDebug("Prepared device: " + device);

   //int oldStyleFileHandle = track.handle();

   // rewind
   track.at(0);
   while(!track.atEnd())
   {
      length = track.readBlock((char*)soundBuffer, 1024);
      //length = ::read( oldStyleFileHandle, soundBuffer, 1024 ); //read from the file
      ::write( dsp, soundBuffer, length); //write to the device
   }

   ::close (dsp);
   track.close();
   qDebug("Finished playing wav file: " + sound + " on device: " + device);

     return true;
}
